@@ -1,13 +1,13 @@
 /*
- * FREE 2000
+ * GNU.FREE 2001
  *
- * Copyright (c) Jason Kitcat, University of Warwick
+ * Copyright (c) 1999, 2000, 2001 The Free Software Foundation (www.fsf.org)
  *
- * jeep@thecouch.org   http://www.thecouch.org/free/
+ * GNU.FREE Co-ordinator: Jason Kitcat <jeep@thecouch.org>
  *
- * Knoll Cottage, Sutton Place, Nr.Abinger, Dorking, Surrey RH5 6RN United Kingdom
- *
- * Tel: 07956 886 508  Fax: 0870 052 7114
+ * GNU site: http://www.gnu.org/software/gnu.free/gnufree.html
+ * 
+ * FREE e-democracy site: http://www.thecouch.org/free/
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,6 +32,20 @@ import java.sql.*;
 import java.util.*;
 import org.hsql.*;
 
+import cryptix.provider.key.RawSecretKey;
+import cryptix.provider.padding.OneAndZeroes;
+import cryptix.provider.cipher.Blowfish;
+import cryptix.util.core.Hex;
+
+import xjava.security.Cipher;
+import xjava.security.FeedbackCipher;
+import xjava.security.SecretKey;
+
+import java.security.*;
+
+import Free.util.StringByteTools;
+import Free.DBPool.*;
+
 /**
  * DBase creates databases, tables and interacts with these through JDBC and SQL.
  *
@@ -39,13 +53,15 @@ import org.hsql.*;
  * <a href="http://hsql.oron.ch">Hypersonic SQL</a> which is written
  * totally in Java. The logic for calculating totals is also performed in this class.
  * 
- * @version 0.12  11 November 2000
+ * @version 1.3  26 February 2001
  * @author Jason Kitcat
  */
 public class DBase {
 
 	private static final String username = "sa";
 	private static final String password = "";
+	//USER SUPPLIED SESSION WORD
+	protected static String cryptword = "";
 	
 
 	/**
@@ -58,10 +74,10 @@ public class DBase {
 
 			Class.forName("org.hsql.jdbcDriver"); // Load the Hypersonic SQL JDBC driver
 
-			/* Connect to the database */
-			/* It will be created automatically if it does not yet exist */
-			Connection freeConn = DriverManager.getConnection("jdbc:HypersonicSQL:rtserver",username, password);
+			new JDCConnectionDriver("org.hsql.jdbcDriver","jdbc:HypersonicSQL:rtserver",username, password);
 			
+			/* Get a connection from the pool */
+			Connection freeConn = getConnection();
 			Statement freeStat = freeConn.createStatement();
 			
 			/* make table for storing votes */
@@ -69,9 +85,13 @@ public class DBase {
 				try {
 					ResultSet r = freeStat.executeQuery("SELECT choice FROM Votes"); //check for the table
 				} catch (SQLException sqle) {
-					RTServer.DEV.debug("No Votes table so creating");
+					if(RTServer.DEV.isDebugEnabled()) {
+						RTServer.DEV.debug("No Votes table so creating");
+					}
 					freeStat.execute("CREATE TABLE Votes(choice VARCHAR(255))");
-					RTServer.DEV.debug("No Keys table so creating");
+					if(RTServer.DEV.isDebugEnabled()) {
+						RTServer.DEV.debug("No Keys table so creating");
+					}
 					freeStat.execute("CREATE TABLE Keys(ak VARCHAR(100))");
 					// now import keys from file
 					importKeys();
@@ -80,14 +100,27 @@ public class DBase {
 				try {
 					ResultSet r = freeStat.executeQuery("SELECT choice FROM Totals"); //check for the table
 				} catch (SQLException sqle) {
-					RTServer.DEV.debug("No Totals table so creating");
-					freeStat.execute("CREATE TABLE Totals(choice VARCHAR(255), total INTEGER)");
+					if(RTServer.DEV.isDebugEnabled()) {
+						RTServer.DEV.debug("No Totals table so creating");
+					}
+					freeStat.execute("CREATE TABLE Totals(choice VARCHAR(255), total VARCHAR(12))");
 				}
 			}
 			
+			DatabaseMetaData dbMetaData = freeConn.getMetaData();
+			
 			freeStat.close();
 			
-			RTServer.NORM.info("Database initialised");
+			RTServer.NORM.info("Database initialised.");
+			RTServer.DEV.debug("DB Using: "  + dbMetaData.getDatabaseProductName() + " " + dbMetaData.getDatabaseProductVersion());
+			
+			// initialise crypto
+		 	try {
+				java.security.Security.addProvider(new cryptix.provider.Cryptix());
+			} catch(Exception e) {
+				RTServer.NORM.error("DBase crypto initialistion error: " + e.toString());
+			}
+
 
 		} catch(Exception e) {
 			RTServer.NORM.error("Database error: " + e.toString());
@@ -96,6 +129,62 @@ public class DBase {
 	} //EOF init()
 
 
+	/**
+	 * Provides a database connection from the pool.
+	 *
+	 * @returns  A working connection
+	 */
+	private static Connection getConnection() throws SQLException {
+        return DriverManager.getConnection("jdbc:jdc:jdcpool");
+    }
+    
+	
+	/**
+	 * Encrypts string data before being inserted into database
+	 *
+	 * @param data  The string to encrypt
+	 * @returns  Encrypted string
+	 */
+	 private static String encrypt(String data) throws Exception {
+	 
+	 	RawSecretKey key;
+	 	byte[] ect;
+		Cipher blowalg = Cipher.getInstance(new Blowfish(),null,new OneAndZeroes());
+	 	
+		/* build a Blowfish (16-round) key using ECB */
+		key = new RawSecretKey("Blowfish", Hex.fromString(cryptword));
+        
+		blowalg.initEncrypt(key);
+		ect = blowalg.crypt(StringByteTools.asciiGetBytes(data));
+
+		return Hex.toString(ect);
+
+	} //eof encrypt()
+
+
+	/**
+	 * Decrypts data pulled from the database returning normal strings.
+	 *
+	 * @param data  The encrypted string
+	 * @returns  Decrypted string
+	 */
+	protected static String decrypt(String data) throws Exception {
+
+		RawSecretKey key;
+	 	byte[] dect;
+		Cipher blowalg = Cipher.getInstance(new Blowfish(),null,new OneAndZeroes());
+	 	
+		/* build a Blowfish (16-round) key using ECB */
+		key = new RawSecretKey("Blowfish", Hex.fromString(cryptword));
+        
+		blowalg.initDecrypt(key);
+		dect = blowalg.crypt(Hex.fromString(data));
+
+		return new String(dect);
+
+	} //eof decrypt()
+
+
 	/**
 	 * checks String input to make sure it only contains safe characters.
 	 * It's not pretty but regular expression packages tried all were far
@@ -104,7 +193,7 @@ public class DBase {
 	 * @param input  The String to check
 	 * @returns  True if the string is ok
 	 */
-	 private static boolean isSafe(String input) throws Exception{
+	 protected static boolean isSafe(String input) throws Exception {
 	 
 	 	boolean ok = true;
 	 	 
@@ -208,24 +297,21 @@ public class DBase {
 	
 		int result = 0;
 	
-		Class.forName("org.hsql.jdbcDriver"); // Load the Hypersonic SQL JDBC driver
-
 		/* boundary check the data first */
 		// only if the data is alphanumeric, punctuation or spaces do we carry on
 		if (isSafe(data)) {
 
-			/* Connect to the database */
-			Connection freeConn = DriverManager.getConnection("jdbc:HypersonicSQL:rtserver",username, password);
-
+			/* Get a connection from the pool */
+			Connection freeConn = getConnection();
 			Statement freeStat = freeConn.createStatement();
 			
 			/* store the vote */
-			result = freeStat.executeUpdate("INSERT INTO Votes VALUES ('" + data + "')");
+			result = freeStat.executeUpdate("INSERT INTO Votes VALUES ('" + encrypt(data) + "')");
 			freeStat.close();
 			
 		} else {
 		
-			throw new Exception("storeVote - invalid charcter(s) used");
+			throw new Exception("storeVote - invalid character(s) used");
 
 		}
 
@@ -243,11 +329,8 @@ public class DBase {
 		int i = 0;
 		String choice, total;
 	
-		Class.forName("org.hsql.jdbcDriver"); // Load the Hypersonic SQL JDBC driver
-
-		/* Connect to the database */
-		Connection freeConn = DriverManager.getConnection("jdbc:HypersonicSQL:rtserver",username, password);
-
+		/* Get a connection from the pool */
+		Connection freeConn = getConnection();
 		Statement freeStat = freeConn.createStatement();
 		
 		/* parse the data into choice and total */
@@ -257,18 +340,18 @@ public class DBase {
 	
 		choice = data.substring(0,i);
 		total = data.substring(i+1);
-		
+
 		/* boundary check the data first */
 		// only if the data is alphanumeric, punctuation or spaces do we carry on
-		if (isSafe(data)) {
+		if (isSafe(choice)&&isSafe(total)) {
 
 			/* store the data */
-			result = freeStat.executeUpdate("INSERT INTO Totals VALUES ('" + choice + "', " + total + ")");
+			result = freeStat.executeUpdate("INSERT INTO Totals VALUES ('" + encrypt(choice) + "', '" + encrypt(total) + "')");
 			freeStat.close();
 
 		} else {
 		
-			throw new Exception("storeTotal - invalid charcter(s) used");
+			throw new Exception("storeTotal - invalid character(s) used");
 
 		}
 
@@ -284,24 +367,23 @@ public class DBase {
 		ResultSet results, subtotal;
 		Vector parties = new Vector(1);
 		Vector totalData = new Vector(1);
+		String temp = "";
 		long turnout = 0;
 		long t = 0;
 		float p;
 		
 		RTServer.NORM.info("Calculating final results...");
 
-		Class.forName("org.hsql.jdbcDriver"); // Load the Hypersonic SQL JDBC driver
-
-		/* Connect to the database */
-		Connection freeConn = DriverManager.getConnection("jdbc:HypersonicSQL:rtserver",username, password);
-
+		/* Get a connection from the pool */
+		Connection freeConn = getConnection();
 		Statement freeStat = freeConn.createStatement();
 			
 		/* count turnout */
 		results = freeStat.executeQuery("SELECT total FROM Totals");
 		
 		while(results.next()) {
-			turnout += new Long(results.getString(1)).longValue();
+			temp = decrypt(results.getString(1));
+			turnout += new Long(temp).longValue();
 		}
 		
 		RTServer.NORM.info("Turnout: " + turnout + " votes were made.");
@@ -311,15 +393,16 @@ public class DBase {
 		
 		/* find the distinct parties */
 		while(results.next()) {
-			parties.addElement(new String(results.getString(1)));
+			parties.addElement(new String(decrypt(results.getString(1))));
 		}
 		
 		/* count the votes for each party */
 		int i = 0;
 		do {
-			subtotal = freeStat.executeQuery("SELECT total FROM Totals WHERE choice = '" + parties.elementAt(i) + "'");
-			while(subtotal.next()) { 
-				t += new Long(subtotal.getString(1)).longValue();
+			temp = (String) parties.elementAt(i);
+			subtotal = freeStat.executeQuery("SELECT total FROM Totals WHERE choice = '" + encrypt(temp) + "'");
+			while(subtotal.next()) {
+				t += new Long(decrypt(subtotal.getString(1))).longValue();
 			}
 			p = (new Float(t).floatValue() / new Float(turnout).floatValue()) * 100;
 			RTServer.NORM.info(parties.elementAt(i) + " received " + t + " votes. This is " + p + "% of the vote.");
@@ -343,12 +426,10 @@ public class DBase {
 		ResultSet results, subtotal;
 		Vector parties = new Vector(1);
 		Vector totalData = new Vector(1);
+		String temp = "";
 	
-		Class.forName("org.hsql.jdbcDriver"); // Load the Hypersonic SQL JDBC driver
-
-		/* Connect to the database */
-		Connection freeConn = DriverManager.getConnection("jdbc:HypersonicSQL:rtserver",username, password);
-
+		/* Get a connection from the pool */
+		Connection freeConn = getConnection();
 		Statement freeStat = freeConn.createStatement();
 			
 		/* get the choices voted for */
@@ -356,13 +437,14 @@ public class DBase {
 		
 		/* find the distinct parties */
 		while(results.next()) {
-			parties.addElement(new String(results.getString(1)));
+			parties.addElement(new String(decrypt(results.getString(1))));
 		}
 		
 		/* count the votes for each party */
 		int i = 0;
 		do {
-			subtotal = freeStat.executeQuery("SELECT COUNT(choice) FROM Votes WHERE choice = '" + parties.elementAt(i) + "'");
+			temp = (String) parties.elementAt(i);
+			subtotal = freeStat.executeQuery("SELECT COUNT(choice) FROM Votes WHERE choice = '" + encrypt(temp) + "'");
 			while(subtotal.next()) {  // formats results for packets
 				totalData.addElement(new String(parties.elementAt(i) + "-" + subtotal.getString(1)));
 			}
@@ -393,19 +475,16 @@ public class DBase {
 			// only if the data is alphanumeric, punctuation or spaces do we carry on
 			if (isSafe(akey)) {
 			
-				Class.forName("org.hsql.jdbcDriver"); // Load the Hypersonic SQL JDBC driver
-
-				/* Connect to the database */
-				Connection freeConn = DriverManager.getConnection("jdbc:HypersonicSQL:rtserver",username, password);
-
+				/* Get a connection from the pool */
+				Connection freeConn = getConnection();
 				Statement freeStat = freeConn.createStatement();
 	
 				/* replace our key with a string */
-				results = freeStat.executeQuery("UPDATE Keys SET ak='USED' WHERE ak='" + akey + "'");
+				results = freeStat.executeQuery("UPDATE Keys SET ak='" + encrypt("USED") + "' WHERE ak='" + encrypt(akey) + "'");
 				
 			} else {
 			
-				throw new Exception("removeKey - invalid charcter(s) used");
+				throw new Exception("removeKey - invalid character(s) used");
 			
 			}
 			
@@ -431,6 +510,8 @@ public class DBase {
 	 * keys weren't stored encrypted, however I feel this is a necessary security precaution 
 	 * especially when the security of the database is not known in our db independent setup.
 	 *
+	 * Note that since 1.5 all db data is encrypted. Thus keys are twice encrypted.
+	 *
 	 * @param data  The string representing the party to insert
 	 * @returns  True if the AuthKey check was ok
 	 */
@@ -440,26 +521,23 @@ public class DBase {
 		boolean keyOK=false;
 		String decKey1, decKey2;
 	
-		Class.forName("org.hsql.jdbcDriver"); // Load the Hypersonic SQL JDBC driver
-
-		/* Connect to the database */
-		Connection freeConn = DriverManager.getConnection("jdbc:HypersonicSQL:rtserver",username, password);
-
+		/* Get a connection from the pool */
+		Connection freeConn = getConnection();
 		Statement freeStat = freeConn.createStatement();
 
 		/* prepare key2 by decrypting it */
 		decKey2 = AuthKey.decrypt(key2,2);
 			
 		/* list the data */
-		result = freeStat.executeQuery("SELECT ak FROM Keys ORDER BY ak;");
+		result = freeStat.executeQuery("SELECT ak FROM Keys WHERE ak <> '" + encrypt("USED") + "' ORDER BY ak;");
 		
 		/* check answer by going through all keys */
 		while (result.next()&&!keyOK) {
-			decKey1 = AuthKey.decrypt(result.getString(1),1);
+			decKey1 = AuthKey.decrypt(decrypt(result.getString(1)),1);
 			
 			if (decKey1.equals(decKey2)) {
 				// send the key to ERServer
-				RTServer.comms.sendKey(result.getString(1));
+				RTServer.comms.sendKey(decrypt(result.getString(1)));
 				// comms will remove key from database if all ok
 				keyOK = true; // break out
 			}			
@@ -491,10 +569,10 @@ public class DBase {
 			/* boundary check the data first */
 			// only if the data is alphanumeric, punctuation or spaces do we carry on
 			if (isSafe(inData)) {
-				doSQL("INSERT INTO Keys VALUES ('" + AuthKey.decrypt(inData,1) + "');");
+				doSQL("INSERT INTO Keys VALUES ('" + encrypt(AuthKey.decrypt(inData,1)) + "');");
 				inData = in.readLine();
 			} else {
-				throw new Exception("importKeys - invalid charcter(s) used");
+				throw new Exception("importKeys - invalid character(s) used");
 			}
 		}
 	 
@@ -514,11 +592,8 @@ public class DBase {
 		boolean res;
 		int count;
 		
-		Class.forName("org.hsql.jdbcDriver"); // Load the Hypersonic SQL JDBC driver
-
-		/* Connect to the database */
-		Connection freeConn = DriverManager.getConnection("jdbc:HypersonicSQL:rtserver",username, password);
-
+		/* Get a connection from the pool */
+		Connection freeConn = getConnection();
 		Statement freeStat = freeConn.createStatement();
 			
 		/* store the vote */
@@ -550,7 +625,6 @@ public class DBase {
 	 */
 	protected static void doSQL(String sqlCommand) {
 	
-		StringBuffer temp = new StringBuffer();
 		ResultSet results;
 		boolean res;
 		int count;
@@ -563,11 +637,8 @@ public class DBase {
 			
 				RTServer.DEV.info("Executing: " + sqlCommand);
 		
-				Class.forName("org.hsql.jdbcDriver"); // Load the Hypersonic SQL JDBC driver
-
-				/* Connect to the database */
-				Connection freeConn = DriverManager.getConnection("jdbc:HypersonicSQL:rtserver",username, password);
-			
+				/* Get a connection from the pool */
+				Connection freeConn = getConnection();
 				Statement freeStat = freeConn.createStatement();
 			
 				/* execute SQL */
@@ -582,9 +653,9 @@ public class DBase {
 				} else {  // otherwise show results
 					while(results.next()) {
 						if (RTServer.serverType == 'R') {  // regional mode
-							RTServer.frame3.showInfo(results.getString(1));
+							RTServer.frame3.showInfo(decrypt(results.getString(1)));
 						} else { // totaller mode
-							RTServer.frame3.showInfo(results.getString(1) + " " + results.getString(2));
+							RTServer.frame3.showInfo(decrypt(results.getString(1)) + " " + decrypt(results.getString(2)));
 						}
 					}  //FIXME: Improve flexibility as assumes fixed no. of columns
 					RTServer.frame3.showInfo("-DONE-");
